DocumentBuilder

Struct DocumentBuilder 

pub struct DocumentBuilder<'a> { /* private fields */ }
DocumentBuilder is a struct to build an arbitrary valid GraphQL document

// fuzz/fuzz_targets/my_apollo_smith_fuzz_target.rs
#![no_main]

use libfuzzer_sys::fuzz_target;
use arbitrary::Unstructured;
use apollo_smith::DocumentBuilder;

fuzz_target!(|input: &[u8]| {
    let mut u = Unstructured::new(input);
    let document = DocumentBuilder::new(&mut u).build()?;
    let document_str = String::from(document);

    // Your code here...
});

Implementations

Source

impl DocumentBuilder<'_>

pub fn arguments( &mut self, constness: Constness, ) -> ArbitraryResult<Vec<Argument>>

Create an arbitrary vector of Argument

pub fn arguments_with_def( &mut self, args_def: &ArgumentsDef, ) -> ArbitraryResult<Vec<Argument>>

Create an arbitrary vector of Argument given ArgumentsDef

pub fn argument(&mut self, constness: Constness) -> ArbitraryResult<Argument>

Create an arbitrary Argument

pub fn argument_with_def( &mut self, input_val_def: &InputValueDef, ) -> ArbitraryResult<Argument>

Create an arbitrary Argument

pub fn arguments_definition(&mut self) -> ArbitraryResult<ArgumentsDef>

Create an arbitrary ArgumentsDef

Source

impl DocumentBuilder<'_>

pub fn description(&mut self) -> ArbitraryResult<Description>

Create an arbitrary Description

Source

impl DocumentBuilder<'_>

pub fn directives( &mut self, directive_location: DirectiveLocation, ) -> ArbitraryResult<IndexMap<Name, Directive>>

Create an arbitrary vector of Directive

pub fn directive( &mut self, directive_location: DirectiveLocation, ) -> ArbitraryResult<Option<Directive>>

Create an arbitrary Directive given a directive location

pub fn directive_def(&mut self) -> ArbitraryResult<DirectiveDef>

Create an arbitrary DirectiveDef

pub fn directive_locations( &mut self, ) -> ArbitraryResult<IndexSet<DirectiveLocation>>

Create an arbitrary IndexSet of DirectiveLocation

Source

impl DocumentBuilder<'_>

pub fn enum_type_definition(&mut self) -> Result<EnumTypeDef>

Create an arbitrary EnumTypeDef

pub fn choose_enum(&mut self) -> Result<&EnumTypeDef>

Choose an arbitrary EnumTypeDef in existings (already created) enum definitions

pub fn arbitrary_variant<'b>( &mut self, enum_: &'b EnumTypeDef, ) -> Result<&'b Name>

Create an arbitrary variant Name given an enum

pub fn enum_values_definition( &mut self, exclude: &IndexSet<Name>, ) -> Result<IndexSet<EnumValueDefinition>>

Create an arbitrary EnumValueDefinition

Source

impl DocumentBuilder<'_>

pub fn fields_definition( &mut self, exclude: &IndexSet<Name>, ) -> ArbitraryResult<Vec<FieldDef>>

Create an arbitrary list of FieldDef

pub fn field(&mut self, _index: usize) -> ArbitraryResult<Field>

Create an arbitrary Field given an object type

Source

impl DocumentBuilder<'_>

pub fn fragment_definition(&mut self) -> ArbitraryResult<FragmentDef>

Create an arbitrary FragmentDef

pub fn fragment_spread( &mut self, excludes: &mut Vec<Name>, ) -> ArbitraryResult<Option<FragmentSpread>>

Create an arbitrary FragmentSpread, returns None if no fragment definition was previously created

pub fn inline_fragment(&mut self) -> ArbitraryResult<InlineFragment>

Create an arbitrary InlineFragment

pub fn type_condition(&mut self) -> ArbitraryResult<TypeCondition>

Create an arbitrary TypeCondition

Source

impl DocumentBuilder<'_>

pub fn input_object_type_definition( &mut self, ) -> ArbitraryResult<InputObjectTypeDef>

Create an arbitrary InputObjectTypeDef

Source

impl DocumentBuilder<'_>

pub fn input_value( &mut self, constness: Constness, ) -> ArbitraryResult<InputValue>

Create an arbitrary InputValue

pub fn input_value_for_type(&mut self, ty: &Ty) -> ArbitraryResult<InputValue>

pub fn input_values_def( &mut self, directive_location: DirectiveLocation, exclude: &IndexSet<Name>, self_name: Option<&Name>, ) -> ArbitraryResult<Vec<InputValueDef>>

Create an arbitrary list of InputValueDef. The caller passes directive_location so directives applied to each value are filtered against the right context — InputFieldDefinition for input-object fields, ArgumentDefinition for argument lists on fields and directives.

pub fn input_value_def( &mut self, directive_location: DirectiveLocation, ) -> ArbitraryResult<InputValueDef>

Create an arbitrary InputValueDef. The caller passes directive_location for the same reason as input_values_def.

Source

impl DocumentBuilder<'_>

pub fn interface_type_definition(&mut self) -> ArbitraryResult<InterfaceTypeDef>

Create an arbitrary InterfaceTypeDef

pub fn implements_interfaces(&mut self) -> ArbitraryResult<IndexSet<Name>>

Pick a random set of interfaces to implement, expanded to its transitive closure so the validator’s “must also implement” rule is satisfied. A candidate is dropped if its closure would conflict with parents already picked.

Source

impl DocumentBuilder<'_>

pub fn name(&mut self) -> ArbitraryResult<Name>

Create an arbitrary Name

pub fn type_name(&mut self) -> ArbitraryResult<Name>

Create an arbitrary type Name that does not yet exist in the document.

pub fn name_with_index(&mut self, index: usize) -> ArbitraryResult<Name>

Create an arbitrary Name with an index included in the name (to avoid name conflict)

Source

impl DocumentBuilder<'_>

pub fn object_type_definition(&mut self) -> ArbitraryResult<ObjectTypeDef>

Create an arbitrary ObjectTypeDef

Source

impl DocumentBuilder<'_>

pub fn operation_definition(&mut self) -> ArbitraryResult<Option<OperationDef>>

Create an arbitrary OperationDef taking the last SchemaDef.

Source

impl DocumentBuilder<'_>

pub fn scalar_type_definition(&mut self) -> ArbitraryResult<ScalarTypeDef>

Create an arbitrary ScalarTypeDef

Source

impl DocumentBuilder<'_>

pub fn schema_definition(&mut self) -> ArbitraryResult<SchemaDef>

Create an arbitrary SchemaDef

Source

impl DocumentBuilder<'_>

pub fn selection_set(&mut self) -> ArbitraryResult<SelectionSet>

Create an arbitrary SelectionSet

pub fn selection( &mut self, index: usize, excludes: &mut Vec<Name>, ) -> ArbitraryResult<Selection>

Create an arbitrary Selection

Source

impl DocumentBuilder<'_>

pub fn ty(&mut self) -> ArbitraryResult<Ty>

Create an arbitrary Ty

pub fn choose_ty(&mut self, existing_types: &[Ty]) -> ArbitraryResult<Ty>

Choose an arbitrary existing Ty given a slice of existing types

pub fn choose_named_ty(&mut self, existing_types: &[Ty]) -> ArbitraryResult<Ty>

Choose an arbitrary existing named Ty given a slice of existing types

Source

impl DocumentBuilder<'_>

pub fn union_type_definition(&mut self) -> ArbitraryResult<UnionTypeDef>

Create an arbitrary UnionTypeDef

Source

impl DocumentBuilder<'_>

pub fn variable_definitions(&mut self) -> ArbitraryResult<Vec<VariableDef>>

Create an arbitrary list of VariableDef

pub fn variable_definition(&mut self) -> ArbitraryResult<VariableDef>

Create an arbitrary VariableDef

Source

impl<'a> DocumentBuilder<'a>

pub fn new(u: &'a mut Unstructured<'a>) -> Self

Create an instance of DocumentBuilder

pub fn max_scalar_types(self, max: usize) -> Self

Set the maximum number of scalar type definitions (default 50).

pub fn max_enum_types(self, max: usize) -> Self

Set the maximum number of enum type definitions (default 50).

pub fn max_interface_types(self, max: usize) -> Self

Set the maximum number of interface type definitions (default 50).

pub fn max_object_types(self, max: usize) -> Self

Set the maximum number of object type definitions (default 50).

pub fn max_union_types(self, max: usize) -> Self

Set the maximum number of union type definitions (default 50).

pub fn max_input_object_types(self, max: usize) -> Self

Set the maximum number of input object type definitions (default 50).

pub fn max_fragment_definitions(self, max: usize) -> Self

Set the maximum number of fragment definitions (default 50).

pub fn max_directive_definitions(self, max: usize) -> Self

Set the maximum number of directive definitions (default 50).

pub fn max_operation_definitions(self, max: usize) -> Self

Set the maximum number of operation definitions (default 50).

pub fn build(self) -> Result<Document>

Generate random definitions according to the configured maximums and return the resulting Document.

pub fn with_document( u: &'a mut Unstructured<'a>, document: Document, ) -> Result<Self>

Create an instance of DocumentBuilder given a Document to be able to call methods on DocumentBuilder and generate valid entities like for example an operation

pub fn input_exhausted(&self) -> bool

Returns whether the provided Unstructured is now empty